#include "msioHL.h"
#include "msioLL.h"
#include "timer.h"

bool msioHlSetRegAddrs(uint8_t firstRead, uint8_t numRead, uint8_t firstWrite, uint8_t numWrite)
{
	uint8_t data[] = {firstRead, numRead, firstWrite, numWrite};
	
	return msioLlTpcW(MS_SET_RW_REG_ADRS, data, sizeof(data));
}

bool msioHlRegsRead(uint8_t *out, uint8_t numRegs)
{
	return msioLlTpcR(MS_RD_REG, out, numRegs);
}

bool msioHlRegsWrite(const uint8_t *data, uint8_t numRegs)
{
	return msioLlTpcW(MS_WR_REG, data, numRegs);
}

bool msioHlSingleRegRead(uint8_t reg, uint8_t *out)
{
	return msioHlSetRegAddrs(reg, 1, reg, 1) && msioHlRegsRead(out, 1);
}

bool msioHlSingleRegWrite(uint8_t reg, uint8_t val)
{
	return msioHlSetRegAddrs(reg, 1, reg, 1) && msioHlRegsWrite(&val, 1);
}

bool msioHlCmd(uint8_t cmd)
{
	return msioLlTpcW(MS_SET_CMD, &cmd, 1);
}

bool msioHlCmdEx(uint8_t cmd, uint32_t addr, uint16_t count)
{
	uint8_t data[] = {cmd, count >> 8, (uint8_t)count, addr >> 24, addr >> 16, addr >> 8, (uint8_t)addr};
	
	return msioLlTpcW(MS_EX_SET_CMD, data, sizeof(data));
}

bool msioHlReadIntReg(uint8_t *intRegValP)
{
	return msioLlTpcR(MS_GET_INT, intRegValP, 1);
}

bool msioHlCmdEndWaitInt(uint32_t maxWaitTicks, uint8_t *intRegValP)
{
	uint64_t startTime = timerGetTicks();
	
	do {
		
		if (msioLlGetInt())
			return intRegValP ? msioHlReadIntReg(intRegValP) : true;
		
	} while (timerGetTicks() - startTime <= maxWaitTicks);
	
	if (intRegValP)
		(void)msioHlReadIntReg(intRegValP);
	return false;
}

bool msioHlCmdEndWaitBusy(uint32_t maxWaitTicks, uint8_t *intRegValP)
{
	uint64_t startTime = timerGetTicks();
	uint8_t intVal;

	if (!intRegValP)
		intRegValP = &intVal;
	
	do {
		
		if (!msioHlReadIntReg(intRegValP))
			return false;
		
		if (*intRegValP)
			return true;
		
	} while (timerGetTicks() - startTime <= maxWaitTicks);
	
	if (intRegValP)
		(void)msioHlReadIntReg(intRegValP);
	return false;
}